#!/usr/bin/env python3
"""
________________________________________________________________________

:PROJECT: SiLA2_python

*RegloICCService*

:details: RegloICCService:
    This is a RegloICC Service
           
:file:    RegloICCService_server.py
:authors: Valeryia Sidarava & Lukas Bromig

:date: (creation)          2020-04-16T10:19:58.324885
:date: (last modification) 2020-04-16T10:19:58.324885

.. note:: Code generated by sila2codegenerator 0.2.0

________________________________________________________________________

**Copyright**:
  This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
  INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

  For further Information see LICENSE file that comes with this distribution.
________________________________________________________________________
"""
__version__ = "1.0"

import os
import logging
import argparse

# Import the main SiLA library
from sila2lib.sila_server import SiLA2Server

# Import gRPC libraries of features
from .CalibrationServicer.gRPC import CalibrationServicer_pb2
from .CalibrationServicer.gRPC import CalibrationServicer_pb2_grpc
# import default arguments for this feature
from .CalibrationServicer.CalibrationServicer_default_arguments import default_dict as CalibrationServicer_default_dict
from .ConfigurationServicer.gRPC import ConfigurationServicer_pb2
from .ConfigurationServicer.gRPC import ConfigurationServicer_pb2_grpc
# import default arguments for this feature
from .ConfigurationServicer.ConfigurationServicer_default_arguments import default_dict as ConfigurationServicer_default_dict
from .DeviceServicer.gRPC import DeviceServicer_pb2
from .DeviceServicer.gRPC import DeviceServicer_pb2_grpc
# import default arguments for this feature
from .DeviceServicer.DeviceServicer_default_arguments import default_dict as DeviceServicer_default_dict
from .DriveControlServicer.gRPC import DriveControlServicer_pb2
from .DriveControlServicer.gRPC import DriveControlServicer_pb2_grpc
# import default arguments for this feature
from .DriveControlServicer.DriveControlServicer_default_arguments import default_dict as DriveControlServicer_default_dict
from .ParameterControlServicer.gRPC import ParameterControlServicer_pb2
from .ParameterControlServicer.gRPC import ParameterControlServicer_pb2_grpc
# import default arguments for this feature
from .ParameterControlServicer.ParameterControlServicer_default_arguments import default_dict as ParameterControlServicer_default_dict

# Import the servicer modules for each feature
from .CalibrationServicer.CalibrationServicer_servicer import CalibrationServicer
from .ConfigurationServicer.ConfigurationServicer_servicer import ConfigurationServicer
from .DeviceServicer.DeviceServicer_servicer import DeviceServicer
from .DriveControlServicer.DriveControlServicer_servicer import DriveControlServicer
from .ParameterControlServicer.ParameterControlServicer_servicer import ParameterControlServicer

import time
import serial
import glob
from .SerialDetector import serial_ports

class RegloICCServiceServer(SiLA2Server):
    """
    This is a RegloICC Service
    """

    def __init__(self, cmd_args, ip: str = '127.0.0.1', port: int = 50001, simulation_mode: bool = False):
        """Class initializer"""
        super().__init__(name=cmd_args.server_name,
                         description=cmd_args.description,
                         server_type=cmd_args.server_type,
                         server_uuid=None,
                         version=__version__,
                         vendor_url="valeryia.sidarava@tum.de",
                         ip=ip, port=port,
                         key_file=cmd_args.encryption_key,
                         cert_file=cmd_args.encryption_cert
                         )

        logging.info(
            "Starting SiLA2 server with server name: {server_name}".format(
                server_name=cmd_args.server_name
            )
        )

        self.simulation_mode = simulation_mode

        if not self.simulation_mode:
            ports_required = glob.glob('/dev/tty[U][S][B]*')[0]

            # Opening a serial port for data communication automatically. ONLY use if no other COM ports exist!
            available_ports = serial_ports()
            ports = []
            ports_not_required = []

            self.ser = []
            for i in range(0, len(available_ports), 1):
                if available_ports[i] in ports_required:
                    ports.append(available_ports[i])
                else:
                    ports_not_required.append(available_ports[i])
            unavailable_ports = list(set(ports_required) - set(ports))
            if unavailable_ports is not []:
                logging.info(
                    "The following COM/USB ports are not available: {ports}".format(ports=unavailable_ports))
            if ports == []:
                logging.info(
                    "No available COM/USB port to establish connection: {ports}".format(ports=ports))
            else:
                ports = ports_required
                for i in range(0, len(unavailable_ports), 1):
                    ports.remove(unavailable_ports[i])
                print(ports)
                self.ser = list()
                for i in range(0, len(ports), 1):
                    print("Connecting to port: {port} as channel_{ID}".format(port=ports[i], ID=i))
                    vars()['channel%s' % i] = serial.Serial(port=ports[i], baudrate=9600,
                                                            parity=serial.PARITY_NONE,
                                                            stopbits=serial.STOPBITS_ONE, bytesize=serial.EIGHTBITS,
                                                            timeout=0.1)
                    self.ser.append(vars()['channel%s' % i])
                time.sleep(2)  # Upon startup, the sensor bars must not be addressed for 1,5 seconds.
                logging.info(
                    "Connection established on ports: {ports}".format(ports=self.ser))

            # Opening a serial port for data communication
            # Not available on BeagleBone/Debian
            # ports=glob.glob('/dev/tty[U][S][B]*')
            # print("Using serial port: %s" %ports[0])
            # self.ser = serial.Serial(port=ports[0], baudrate=9600, parity=serial.PARITY_NONE, stopbits=serial.STOPBITS_ONE,bytesize=8, timeout=1, xonxoff=0)
        else:
            self.ser = []
        
        # registering features
        
        #  Register CalibrationServicer
        self.CalibrationServicer_servicer = CalibrationServicer(ser=self.ser,
                                                                simulation_mode=self.simulation_mode)
        CalibrationServicer_pb2_grpc.add_CalibrationServicerServicer_to_server(
            self.CalibrationServicer_servicer,
            self.grpc_server
        )
        self.add_feature(feature_id='CalibrationServicer',
                         servicer=self.CalibrationServicer_servicer,
                         data_path=os.path.join(
                             os.path.dirname(__file__),
                             'meta')
                         )
                         
        #  Register ConfigurationServicer
        self.ConfigurationServicer_servicer = ConfigurationServicer(ser=self.ser,
                                                                    simulation_mode=self.simulation_mode)
        ConfigurationServicer_pb2_grpc.add_ConfigurationServicerServicer_to_server(
            self.ConfigurationServicer_servicer,
            self.grpc_server
        )
        self.add_feature(feature_id='ConfigurationServicer',
                         servicer=self.ConfigurationServicer_servicer,
                         data_path=os.path.join(
                             os.path.dirname(__file__),
                             'meta')
                         )
                         
        #  Register DeviceServicer
        self.DeviceServicer_servicer = DeviceServicer(ser=self.ser,
                                                      simulation_mode=self.simulation_mode)
        DeviceServicer_pb2_grpc.add_DeviceServicerServicer_to_server(
            self.DeviceServicer_servicer,
            self.grpc_server
        )
        self.add_feature(feature_id='DeviceServicer',
                         servicer=self.DeviceServicer_servicer,
                         data_path=os.path.join(
                             os.path.dirname(__file__),
                             'meta')
                         )
                         
        #  Register DriveControlServicer
        self.DriveControlServicer_servicer = DriveControlServicer(ser=self.ser,
                                                                  simulation_mode=self.simulation_mode)
        DriveControlServicer_pb2_grpc.add_DriveControlServicerServicer_to_server(
            self.DriveControlServicer_servicer,
            self.grpc_server
        )
        self.add_feature(feature_id='DriveControlServicer',
                         servicer=self.DriveControlServicer_servicer,
                         data_path='meta')
                         
        #  Register ParameterControlServicer
        self.ParameterControlServicer_servicer = ParameterControlServicer(ser=self.ser,
                                                                          simulation_mode=self.simulation_mode)
        ParameterControlServicer_pb2_grpc.add_ParameterControlServicerServicer_to_server(
            self.ParameterControlServicer_servicer,
            self.grpc_server
        )
        self.add_feature(feature_id='ParameterControlServicer',
                         servicer=self.ParameterControlServicer_servicer,
                         data_path=os.path.join(
                             os.path.dirname(__file__),
                             'meta')
                         )
                         
        self.simulation_mode = simulation_mode
        
        
        # starting and running the gRPC/SiLA2 server
        self.run()
        for ser in self.ser:
            if ser:
                ser.close()

        logging.debug("connection closed")

    def switchToSimMode(self):
        #Switch to simulation mode of the server

        # perform implementation specific switch operations
        self.DeviceServicer_servicer.switch_to_simulation_mode()
        self.ParameterControlServicer_servicer.switch_to_simulation_mode()
        self.CalibrationServicer_servicer.switch_to_simulation_mode()
        self.ConfigurationServicer_servicer.switch_to_simulation_mode()
        self.DriveControlServicer_servicer.switch_to_simulation_mode()

        self.simulation_mode = True

    def switchToRealMode(self):
        #Switch to real mode

        # perform implementation specific switch operations
        self.DeviceServicer_servicer.switch_to_real_mode()
        self.ParameterControlServicer_servicer.switch_to_real_mode()
        self.CalibrationServicer_servicer.switch_to_real_mode()
        self.ConfigurationServicer_servicer.switch_to_real_mode()
        self.DriveControlServicer_servicer.switch_to_real_mode()

        self.simulation_mode = False


def parse_command_line():
    """
    Just looking for commandline arguments
    """
    parser = argparse.ArgumentParser(description="A SiLA2 service: RegloICCService")

    # Simple arguments for the server identification
    parser.add_argument('-s', '--server-name', action='store',
                        default="RegloICCService", help='start SiLA server with [server-name]')
    parser.add_argument('-t', '--server-type', action='store',
                        default="Unknown Type", help='start SiLA server with [server-type]')
    parser.add_argument('-d', '--description', action='store',
                        default="This is a RegloICC Service", help='SiLA server description')

    # Encryption
    parser.add_argument('-X', '--encryption', action='store', default=None,
                        help='The name of the private key and certificate file (without extension).')
    parser.add_argument('--encryption-key', action='store', default=None,
                        help='The name of the encryption key (*with* extension). Can be used if key and certificate '
                             'vary or non-standard file extensions are used.')
    parser.add_argument('--encryption-cert', action='store', default=None,
                        help='The name of the encryption certificate (*with* extension). Can be used if key and '
                             'certificate vary or non-standard file extensions are used.')

    parser.add_argument('-v', '--version', action='version', version='%(prog)s ' + __version__)

    parsed_args = parser.parse_args()

    # validate/update some settings
    #   encryption
    if parsed_args.encryption is not None:
        # only overwrite the separate keys if not given manually
        if parsed_args.encryption_key is None:
            parsed_args.encryption_key = parsed_args.encryption + '.key'
        if parsed_args.encryption_cert is None:
            parsed_args.encryption_cert = parsed_args.encryption + '.cert'

    return parsed_args
    
        
if __name__ == '__main__':
    # or use logging.ERROR for less output
    logging.basicConfig(format='%(levelname)-8s| %(module)s.%(funcName)s: %(message)s', level=logging.DEBUG)
    
    args = parse_command_line()

    # generate SiLA2Server
    sila_server = RegloICCServiceServer(cmd_args=args, ip='127.0.0.1', port=50001, simulation_mode=True)
